@@ -117,13 +117,13 @@ class EbuildFetcher(ForkProcess):
 			msg_lines.append(msg)
 			self._eerror(msg_lines)
 			self._set_returncode((self.pid, 1 << 8))
-			self.wait()
+			self._async_wait()
 			return
 
 		if not uri_map:
 			# Nothing to fetch.
 			self._set_returncode((self.pid, os.EX_OK << 8))
-			self.wait()
+			self._async_wait()
 			return
 
 		settings = self.config_pool.allocate()
@@ -135,7 +135,7 @@ class EbuildFetcher(ForkProcess):
 			self._prefetch_size_ok(uri_map, settings, ebuild_path):
 			self.config_pool.deallocate(settings)
 			self._set_returncode((self.pid, os.EX_OK << 8))
-			self.wait()
+			self._async_wait()
 			return
 
 		nocolor = settings.get("NOCOLOR")
@@ -165,7 +165,8 @@ class EbuildFetcher(ForkProcess):
 			not in ('yes', 'true')
 
 		rval = 1
-		allow_missing = self._get_manifest().allow_missing
+		allow_missing = self._get_manifest().allow_missing or \
+			'digest' in self._settings.features
 		if fetch(self._uri_map, self._settings, fetchonly=self.fetchonly,
 			digests=copy.deepcopy(self._get_digests()),
 			allow_missing_digests=allow_missing):
