@@ -455,6 +455,16 @@ def parse_opts(tmpcmdline, silent=False):
 			"choices": y_or_n
 		},
 
+		"--ignore-soname-deps": {
+			"help": "Ignore the soname dependencies of binary and "
+				"installed packages. This option is enabled by "
+				"default, since soname dependencies are relatively "
+				"new, and the required metadata is not guaranteed to "
+				"exist for binary and installed packages built with "
+				"older versions of portage.",
+			"choices": y_or_n
+		},
+
 		"--jobs": {
 
 			"shortopt" : "-j",
@@ -1100,6 +1110,29 @@ def emerge_main(args=None):
 		# loading to allow new repos with non-existent directories
 		portage._sync_mode = True
 
+	# Verify that /dev/null exists and is a device file as a cheap early
+	# filter for obviously broken /dev/s.
+	try:
+		if os.stat(os.devnull).st_rdev == 0:
+			writemsg_level("Failed to validate a sane '/dev'.\n"
+				  "'/dev/null' is not a device file.\n",
+				  level=logging.ERROR, noiselevel=-1)
+			return 1
+	except OSError:
+		writemsg_level("Failed to validate a sane '/dev'.\n"
+				 "'/dev/null' does not exist.\n",
+				 level=logging.ERROR, noiselevel=-1)
+		return 1
+
+	# Verify that BASH process substitution works as another cheap early
+	# filter. Process substitution uses '/dev/fd'.
+	if portage.process.spawn_bash("[[ $(< <(echo foo) ) == foo ]]") != 0:
+		writemsg_level("Failed to validate a sane '/dev'.\n"
+				 "bash process substitution doesn't work; this may be an "
+				 "indication of a broken '/dev/fd'.\n",
+				 level=logging.ERROR, noiselevel=-1)
+		return 1
+
 	# Portage needs to ensure a sane umask for the files it creates.
 	os.umask(0o22)
 	emerge_config = load_emerge_config(
